//*--------------------------------------------------------------------------------------*/
/*	Copyright (C) M-System Co., Ltd.  (2004). All Rights Reserved	  */
/*	Device:		            B3PU for PROFIBUS PA	                         */
/*	Description:                                        					                */
/*	Dictionary:	           SIPADC01.dct						               */
/*                                                         		                                                */
/*  Revision:                  1.00  						                                */
/*  Date:                        June  29, 2004 			                                 */
/* Author:                      Tianbing Li                                                                  */
/*--------------------------------------------------------------------------------------*/

/**Definations used in functions extended for user-defined linearization table **/
#define TB_SPEC_OPERATION_CODE_INITIAL		 02
#define TB_SPEC_OPERATION_CODE_EDITABLE	    165
#define TB_SPEC_OPERATION_CODE_OK		    00
#define TB_SPEC_SENSOR_LIN_TYPE_RTD		   240
#define TB_SPEC_SENSOR_LIN_TYPE_TC		     241
#define TEMPERATURE_UNIT_KELVIN			          1000
#define TEMPERATURE_UNIT_CELSIUS		        1001
#define TEMPERATURE_UNIT_FAHRENHEIT		    1002
#define TEMPERATURE_UNIT_RANKINE		        1003
/*******************************************************************************************/


#define TRANS1_LOAD_TO_PC						    		\
		trans1_st_rev,										\
		trans1_lin_type,									\
		trans1_primary_value_unit,							\
		trans1_sensorscalefactor,							\
		trans1_sensor_measure_type,							\
		trans1_input_range,									\
		trans1_bias_1,										\
		trans1_reference_junction_type,						\
		trans1_sensor_connection,							\
		trans1_comp_wire_1,									\
	     	trans1_external_reference_junction_value,			\
		trans1_referenceconnection,							\
		trans1_comp_wire_2,									\
		trans1_reference_comp_wire,							\
		trans1_measurecompwire,								\
		trans1_sensor_wire_check_1,							\
	     	trans1_sensorfilter,								\
		trans1_lower_sensor_limit,                                              \
		trans1_upper_sensor_limit                                                 \
		tb_spec_lin_type,		                                              \
		tb_spec_max_number,                                                          \
		tb_spec_temperature_unit,                                                    \
		tb_spec_min_temperature,                                     \
		tb_spec_pitch,                                            \
		tb_spec_actual_index_number,                    \
		tb_spec_index,                                             \
		tb_spec_entry_x,                         \
		tb_spec_operation_code  ,       \
                tb_sensor_Lower_trim,               \
                tb_sensor_Upper_trim


#define TRANS1_LOAD_TO_DEVICE								\
		trans1_lin_type,									\
		trans1_primary_value_unit,							\
		trans1_sensorscalefactor,							\
		trans1_sensor_measure_type,							\
		trans1_input_range,									\
		trans1_bias_1,										\
		trans1_reference_junction_type,						\
		trans1_sensor_connection,							\
		trans1_comp_wire_1,									\
		trans1_bias_2,										\
		trans1_external_reference_junction_value,			\
		trans1_referenceconnection,							\
		trans1_comp_wire_2,									\
		trans1_reference_comp_wire,							\
		trans1_measurecompwire,								\
		trans1_sensor_wire_check_1,							\
		trans1_sensor_wire_check_2	           		

#define DEF_LIN_TYPE_RTD        ((trans1_lin_type >= 100) && (trans1_lin_type <= 127))      \
                                                                || trans1_lin_type == TB_SPEC_SENSOR_LIN_TYPE_RTD
#define DEF_INPUT_RANGE_MV      (trans1_input_range >= 0) && (trans1_input_range <= 6)

#define MIN_trans1_bias_1					\
		SELECT(trans1_primary_value_unit)	\
		{									\
			CASE 1000: -10;					\
			CASE 1001: -10;					\
			CASE 1002: -18;					\
			DEFAULT: -10;					\
		}									\

#define MAX_trans1_bias_1					\
		SELECT(trans1_primary_value_unit)	\
		{									\
			CASE 1000: 10;					\
			CASE 1001: 10;					\
			CASE 1002: 18;					\
			DEFAULT: 10;					\
		}									\

IMPORT "MSYS_B3PU_00_trans1.imp"
{
	EVERYTHING;

	REDEFINITIONS
	{
		
		
		DELETE COMMAND read_trans1_min_sensor_value_1;
		DELETE COMMAND write_trans1_min_sensor_value_1;

		DELETE COMMAND read_trans1_max_sensor_value_1;
		DELETE COMMAND write_trans1_max_sensor_value_1;
		DELETE COMMAND read_trans1_min_sensor_value_2;
		DELETE COMMAND write_trans1_min_sensor_value_2;

		DELETE COMMAND read_trans1_max_sensor_value_2;
		DELETE COMMAND write_trans1_max_sensor_value_2;

		DELETE VARIABLE trans1_min_sensor_value_1;
		DELETE VARIABLE trans1_max_sensor_value_1;
		DELETE VARIABLE trans1_min_sensor_value_2;
		DELETE VARIABLE trans1_max_sensor_value_2;
		DELETE VARIABLE trans1_bias_2; 
		DELETE VARIABLE trans1_comp_wire_2;
		DELETE VARIABLE trans1_sensor_wire_check_2; 
		DELETE VARIABLE trans1_secondary_value_2;
		DELETE VARIABLE trans1_secondary_value_2_status_quality;
		DELETE VARIABLE trans1_secondary_value_2_status_limit;
		DELETE VARIABLE trans1_input_fault_2; 
      
		 
		VARIABLE trans1_primary_value_unit
		{
			REDEFINE TYPE		ENUMERATED (2)
			{
				DEFAULT_VALUE                   
					IF (trans1_lin_type == 0)					
					{
						1243;	//mV
					}
					ELSE
					{
						 1001;  // C
					}
				IF (trans1_lin_type == 0)
				{
						{ 1243,	[unit_1243],	[unit_1243_help]	},/* mV	*/
						{ 1281,	[unit_1281],	[unit_1281_help]	},/* Ohm	*/
						{ 1342,	[unit_1342],	[unit_1342_help]	},/* %	*/
				}
				ELSE
				{	
					/* K	*/          { 1000,	    [unit_1000],	    [unit_1000_help]	    },
					/* degC	*/	{ 1001,	        [unit_1001],	        [unit_1001_help]	},
					/* degF	*/	{ 1002,	        [unit_1002],	        [unit_1002_help]	},
					/* degR	*/	{ 1003,	        [unit_1003],	        [unit_1003_help]	},
				}
			}
		REDEFINE POST_EDIT_ACTIONS
			{
				method_set_trans1_primary_value_unit
			}

        } 

		
		VARIABLE trans1_lin_type
		{
			REDEFINE TYPE		ENUMERATED (1)
			{
				DEFAULT_VALUE	0;
				{ 0,	[linear]			},			
				{ 102,	[RTD_Pt100]		}, 
                                { 103,	[RTD_Pt200]		},  
                                { 104,	[RTD_Pt500]		},  
                                { 105,	[RTD_Pt1000]		},   		
				{ 107,	[RTD_Pt50_JIS]		},
				{ 108,	[RTD_Pt100_JIS]		},
				{ 120,	[RTD_Ni120_Edi
]   },	
				{ 126,	[RTD_Cu10]		},   
                                { 240,	[RTD_SPEC]		}, 	
				{ 128,	[TC_type_B]		},
				{ 129,	[TC_type_C]		},
				{ 131,	[TC_type_E]		},
				{ 133,	[TC_type_J]		},
				{ 134,	[TC_type_K]		},
				{ 135,	[TC_type_N]		},
				{ 136,	[TC_type_R]		},
				{ 137,	[TC_type_S]		},
				{ 138,	[TC_type_T]		},
				{ 139,	[TC_type_L]		},
				{ 140,	[TC_type_U]		},
                                { 143,	[TC_Platinel_II]        },
				{ 241,	[TC_SPEC]		}
			}
			ADD POST_EDIT_ACTIONS
			{
				method_set_trans1_lin_type
			}
		}
		VARIABLE trans1_input_range
		{
			REDEFINE TYPE		ENUMERATED (1)
			{
				DEFAULT_VALUE	
					IF (trans1_lin_type == 0)		
					{
                                                SELECT(trans1_primary_value_unit)
                                                {
                                                        CASE    1342:           134;
                                                        CASE    1281:           134;
                                                        CASE    1243:            6;
                                                        DEFAULT:                  6;
                                                }
					}
					ELSE
					{
						IF( DEF_LIN_TYPE_RTD)  					//RTD
						{
							134;
						}
						ELSE
						{
							6;
						}
					}
				IF (trans1_lin_type == 0)		
				{
					IF ( trans1_primary_value_unit == 1243)	 //mV
					{
						{ 0,	[trans_temp_input_range_mV_RANGE_1_msys]		},
						{ 1,	[trans_temp_input_range_mV_RANGE_2_msys]		},
						{ 2,	[trans_temp_input_range_mV_RANGE_3_msys]		},
						{ 3,	[trans_temp_input_range_mV_RANGE_4_msys]		},
						{ 4,	[trans_temp_input_range_mV_RANGE_5_msys]		},
						{ 5,	[trans_temp_input_range_mV_RANGE_6_msys]		},
						{ 6,	[trans_temp_input_range_mV_RANGE_7_msys]		}
					}
					ELSE
					{
						IF ( trans1_primary_value_unit == 1281)	 //Ohm
						{
							{ 128,	[trans_temp_input_range_Ohm_RANGE_1_msys]		},
							{ 129,	[trans_temp_input_range_Ohm_RANGE_2_msys]		},
							{ 130,	[trans_temp_input_range_Ohm_RANGE_3_msys]		},
							{ 131,	[trans_temp_input_range_Ohm_RANGE_4_msys]		},
							{ 132,	[trans_temp_input_range_Ohm_RANGE_5_msys]		},
							{ 133,	[trans_temp_input_range_Ohm_RANGE_6_msys]		},
							{ 134,	[trans_temp_input_range_Ohm_RANGE_7_msys]		}
						}
						ELSE					// %
						{
							{ 129,	[trans_temp_input_range_Ohm_RANGE_2_msys]		},
							{ 130,	[trans_temp_input_range_Ohm_RANGE_3_msys]		},
							{ 131,	[trans_temp_input_range_Ohm_RANGE_4_msys]		},
							{ 132,	[trans_temp_input_range_Ohm_RANGE_5_msys]		},
							{ 133,	[trans_temp_input_range_Ohm_RANGE_6_msys]		},
							{ 134,	[trans_temp_input_range_Ohm_RANGE_7_msys]		}
						}
					}
				}
				ELSE
				{
					IF( DEF_LIN_TYPE_RTD)  					//RTD
					{
						{ 134,	[trans_temp_input_range_Ohm_RANGE_7_msys]		}
					}
					ELSE
					{
						{ 6,	[trans_temp_input_range_mV_RANGE_7_msys]		}
					}
				}
			}

			REDEFINE POST_EDIT_ACTIONS
			{
				method_set_trans1_input_range
			}

		}
 
		VARIABLE trans1_bias_1
		{
			REDEFINE TYPE		FLOAT
			{
				DEFAULT_VALUE	0.0;
				MIN_VALUE		MIN_trans1_bias_1
				MAX_VALUE		MAX_trans1_bias_1
				DISPLAY_FORMAT	"7.3f";	
			}
			ADD CONSTANT_UNIT SELECT (trans1_primary_value_unit)
			{
				CASE 1000:	[unit_1000];
				CASE 1001:	[unit_1001];
				CASE 1002:	[unit_1002];
				CASE 1003:	[unit_1003];
				CASE 1243:	[unit_1243];
				CASE 1281:	[unit_1281];
				CASE 1342:	[unit_1342];
			}
			/*ADD PRE_EDIT_ACTIONS
			{
				method_set_trans1_bias_1
			}*/
		}
		VARIABLE trans1_sensor_measure_type
		{
			REDEFINE TYPE		ENUMERATED (1)
			{
				DEFAULT_VALUE	0;
				{  0,[trans_temp_sensor_measure_type_T1]  }
			}
		}
		
		#if 0
		MENU Tab_s_trans1
		{
			REDEFINE ITEMS
			{
				trans1_lin_type,
				trans1_primary_value_unit,
				trans1_input_range,
				trans1_sensor_measure_type,
				trans1_bias_1, 
				trans1_reference_junction_type,
				trans1_external_reference_junction_value,
				trans1_sensor_connection,
				trans1_comp_wire_1,
				trans1_sensor_wire_check_1,
				
			#ifndef NO_trans1_min_sensor_value_1
					//Tab_s_trans1_measindicator,	// Menu Schleppzeiger
			#endif

				Tab_s_trans1_measrange		// Menu Messgrenzen
				
				trans1_st_rev,
				trans1_tag_desc,
				trans1_strategy,
				trans1_alert_key,
				trans1_target_mode,
				trans1_primary_value,
				trans1_secondary_value_1,
				trans1_input_fault_gen,
				trans1_input_fault_1,	
			}
		}
		#endif
	
		#define TRANS1_VALID_TC							\
				IF(     ((trans1_lin_type >= 128)&&(trans1_lin_type <= 144))         \
                                        || (trans1_lin_type == 241)    ) 			\
				{         								\
				       	TRUE;							\
				}										\
				ELSE									\
				{										\
					FALSE;								\
				}

		#define TRANS1_VALID_VALID_RTD_R  \
				IF(( ( (trans1_lin_type >= 100)&&(trans1_lin_type <= 127) )||(trans1_lin_type == 240) )  /*RDT*/   \
                                        ||( (trans1_lin_type == 0)&&(trans1_primary_value_unit == 1281) )    )   /* R */             \
                           	{	/* RTD or Resistor */							\
					TRUE ;								\
				}										\
				ELSE									\
				{										\
					FALSE ;								\
				}		
		
		VARIABLE trans1_sensor_connection
		{
			REDEFINE VALIDITY	TRANS1_VALID_VALID_RTD_R
     		}
		
		#define TRANS1_VALID_NOT_TC1_trans1_comp_wire_1		\
				IF(   (trans1_sensor_connection == 0)             \
                                        &&  ( (( (trans1_lin_type >= 100)&&(trans1_lin_type <= 127) )||(trans1_lin_type == 240))  /*RDT*/   \
                                                        ||((trans1_lin_type == 0)&&(trans1_primary_value_unit == 1281))    )   /* R */             \       
                                    ) \
				{			\
					TRUE;	\
				}			\
				ELSE		\
				{			\
					FALSE;	\
				}			

		VARIABLE trans1_comp_wire_1
		{
			REDEFINE VALIDITY	TRANS1_VALID_NOT_TC1_trans1_comp_wire_1 
		}

     		VARIABLE trans1_reference_junction_type
		{
			REDEFINE VALIDITY	TRANS1_VALID_TC
		}

 		VARIABLE trans1_external_reference_junction_value
		{
			REDEFINE TYPE		FLOAT
			{
				MIN_VALUE	SELECT(trans1_primary_value_unit)			
							{													
								CASE 1000: 233.15;					
								CASE 1001: -40;						
								CASE 1002: -40;						
							}
				
				MAX_VALUE	SELECT(trans1_primary_value_unit)					
							{													
								CASE 1000: 358.15;					
								CASE 1001: 85;						
								CASE 1002: 185;						
							}

				DEFAULT_VALUE	SELECT(trans1_primary_value_unit)			
								{													
									CASE 1000: 233.15;					
									CASE 1001: 0.0;						
									CASE 1002: 0.0;
									DEFAULT: 0.0;
								}
				
				DISPLAY_FORMAT	"7.3f"; 
			}
			REDEFINE VALIDITY	IF(     (trans1_reference_junction_type == 2)               
                                                                               &&(     ((trans1_lin_type >= 128)&&(trans1_lin_type <= 144))        
                                                                                         || (trans1_lin_type == 241)    )      )
								{
									TRUE;
								}
									ELSE
								{
									FALSE;
								}							
		
 		}					

		VARIABLE trans1_reference_junction_temp
		{
			REDEFINE VALIDITY	TRANS1_VALID_TC
		}            

	}
}

//----------------------METHOD
METHOD method_set_trans1_primary_value_unit
{
	CLASS		INPUT;
	DEFINITION
	{
                int li_typ,pv_unit,in_rang;
                
                li_typ =   ivar_value(trans1_lin_type);
                pv_unit =  ivar_value(trans1_primary_value_unit);
                in_rang  =     ivar_value(trans1_input_range);
                
            //-----------INPUT_RANGE
              if(li_typ == 0)  
              {
                        if(pv_unit==1342)         // %,PotMeter
                        {
                                if( (in_rang<129)  ||    (in_rang>134)   )
                                        trans1_input_range=134;
                        }
                        else if(pv_unit==1243)     //mV
                        {
                                if (in_rang>6 )
                                        trans1_input_range=6;  
                        }
                        else                                                                    //Ohm
                        {
                                if( (in_rang<128)  ||    (in_rang>134)   )
                                        trans1_input_range=134;
                        }
              }
              else        //TC,RTD
              {
                        if(    ((li_typ>= 128)&&(li_typ <= 144))       
                               || (li_typ == 241)    )            //TC
                        {
                                if (in_rang>6 )
                                        trans1_input_range=6; 
                        }
                        else                                            //RTD
                        {
                                if( (in_rang<128)  ||    (in_rang>134)   )
                                        trans1_input_range=134;               
                        }
              }  
           	
        }
}




METHOD  method_set_trans1_input_range
{
	CLASS		INPUT;
	DEFINITION
	{
      	}
} 

METHOD  method_set_trans1_lin_type
{
	LABEL "method_set_trans1_lin_type";
	CLASS		INPUT;
	DEFINITION
	{
                int li_typ,pv_unit,in_rang;
                
                li_typ =   ivar_value(trans1_lin_type);
                pv_unit =  ivar_value(trans1_primary_value_unit);
                in_rang  =     ivar_value(trans1_input_range);
                
                // ------UNIT
                if (li_typ != 0)      //temperature UNIT         
                {
                        if( (pv_unit<1000)||(pv_unit > 1003) )
                                trans1_primary_value_unit=1001;                 //C       
                }
                else
                {
                     if( (pv_unit>=1000)&&(pv_unit <= 1003) )
                                trans1_primary_value_unit=1243;               //mV    
                }           
              //-----------INPUT_RANGE
              pv_unit =  ivar_value(trans1_primary_value_unit);
              if(li_typ == 0)  
              {
                        if(pv_unit==1342)         // %,PotMeter
                        {
                                if( (in_rang<129)  ||    (in_rang>134)   )
                                        trans1_input_range=134;
                        }
                        else if(pv_unit==1243)     //mV
                        {
                                if (in_rang>6 )
                                        trans1_input_range=6;  
                        }
                        else                                                                    //Ohm
                        {
                                if( (in_rang<128)  ||    (in_rang>134)   )
                                        trans1_input_range=134;
                        }
              }
              else        //TC,RTD
              {
                        if(    ((li_typ>= 128)&&(li_typ <= 144))       
                               || (li_typ == 241)    )            //TC
                        {
                                if (in_rang>6 )
                                        trans1_input_range=6; 
                        }
                        else                                            //RTD
                        {
                                if( (in_rang<128)  ||    (in_rang>134)   )
                                        trans1_input_range=134;               
                        }
              }  
              //-----------wires
             // pv_unit =  ivar_value(trans1_primary_value_unit);
              if(li_typ == 0)  
              {
                        if(pv_unit==1342)         // %,PotMeter
                        {
                                trans1_sensor_connection = 1;		//3-wires
                        }
                        else if(pv_unit==1243)     //mV
                        {
                                 trans1_sensor_connection = 0;		//2-wires
                        }
              }
              else        //TC,RTD
              {
                        if(    ((li_typ>= 128)&&(li_typ <= 144))       
                               || (li_typ == 241)    )            //TC
                        {
                               trans1_sensor_connection = 0;		//2-wires  
                        }
              }
	}
}

METHOD method_set_trans1_external_reference_junction_value
{
	CLASS		INPUT;
	DEFINITION
	{
		switch(trans1_primary_value_unit)
		{
			case 1000: trans1_external_reference_junction_value = 233.15;break;
			case 1001: trans1_external_reference_junction_value = -40;break;
			case 1002: trans1_external_reference_junction_value = -40;break;
			default: trans1_external_reference_junction_value = 0.0;break;
		}
	}
}

//Change004
/*METHOD method_set_trans1_bias_1
{
	CLASS		INPUT;
	DEFINITION
	{
		if(trans1_primary_value_unit == 1002)
		{
			
		}
	}
}*/
METHOD method_set_sensor_connection
{
	CLASS		INPUT;
	DEFINITION
	{
		if (trans1_lin_type == 0)
		{		
			primary_value_unit = ivar_value(trans1_primary_value_unit);
			if (primary_value_unit == 1243)					// linear [mV]
			{
					trans1_sensor_connection = 0;		//2-wires
			}
			else if (primary_value_unit == 1281)			// linear [Ohm]
			{
				;
			}		
			else if (primary_value_unit == 1342)			// linear [%]
			{
					trans1_sensor_connection = 1;	// 3-wires
			}						
		}
		// RTD's:
		else if (DEF_LIN_TYPE_RTD)
		{
			;
		}
		// Thermo-Elemente:
		else  if (((ivar_value(trans1_lin_type) >= 128)  && (ivar_value(trans1_lin_type) <= 144)) || (ivar_value(trans1_lin_type) == 241))
		{
			trans1_sensor_connection = 0;		//2-wires
		}
		// Default
		else
		{
			trans1_sensor_connection = 0;		//2-wires
		}

	}
}